﻿using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Runtime;

namespace OpcLabs.BaseLibTest
{
    [TestClass]
    public class ClsTest
    {
        #region Static Private Methods

        static private void TestConvertToClsType([CanBeNull] object value, [CanBeNull] object converted)
        {
            object convertToClsType = Cls.ConvertToClsType(value);
            Assert.AreEqual(converted, convertToClsType);
        }

        static private void TestEqualConvertToClsType([CanBeNull] object value)
        {
            TestConvertToClsType(value, value);
        }

        #endregion



        #region Test Methods
        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void ConvertToClsTypeTest_Byte_0()
        {
            TestEqualConvertToClsType((Byte)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Byte_MinValue()
        {
            TestEqualConvertToClsType(Byte.MinValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Byte_MaxValue()
        {
            TestEqualConvertToClsType(Byte.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Int32_0()
        {
        // ReSharper disable once RedundantCast
            TestEqualConvertToClsType((Int32)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Int32_MinValue()
        {
            TestEqualConvertToClsType(Int32.MinValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Int32_MaxValue()
        {
            TestEqualConvertToClsType(Int32.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Int64_0()
        {
            TestEqualConvertToClsType((Int64)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Int64_MinValue()
        {
            TestEqualConvertToClsType(Int64.MinValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Int64_MaxValue()
        {
            TestEqualConvertToClsType(Int64.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Single_0()
        {
            TestEqualConvertToClsType((Single)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Single_MinValue()
        {
            TestEqualConvertToClsType(Single.MinValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Single_MaxValue()
        {
            TestEqualConvertToClsType(Single.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Double_0()
        {
            TestEqualConvertToClsType((Double)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Double_MinValue()
        {
            TestEqualConvertToClsType(Double.MinValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Double_MaxValue()
        {
            TestEqualConvertToClsType(Double.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Boolean_False()
        {
            TestEqualConvertToClsType(false);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Boolean_True()
        {
            TestEqualConvertToClsType(true);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Char_0()
        {
            TestEqualConvertToClsType((Char)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Char_MinValue()
        {
            TestEqualConvertToClsType(Char.MinValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Char_MaxValue()
        {
            TestEqualConvertToClsType(Char.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Decimal_0()
        {
            TestEqualConvertToClsType((Decimal)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Decimal_MinValue()
        {
            TestEqualConvertToClsType(Decimal.MinValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_Decimal_MaxValue()
        {
            TestEqualConvertToClsType(Decimal.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_IntPtr_0()
        {
            TestEqualConvertToClsType((IntPtr)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_IntPtr_HighValue()
        {
            TestEqualConvertToClsType((IntPtr)Int32.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_IntPtr_LowValue()
        {
            TestEqualConvertToClsType((IntPtr)Int32.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_SByte_0()
        {
            TestConvertToClsType((SByte)0, (Int16)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_SByte_MinValue()
        {
            TestConvertToClsType(SByte.MinValue, (Int16)SByte.MinValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_SByte_MaxValue()
        {
            TestConvertToClsType(SByte.MaxValue, (Int16)SByte.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_UInt16_0()
        {
            // ReSharper disable once RedundantCast
            TestConvertToClsType((UInt16)0, (Int32)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_UInt16_MinValue()
        {
            TestConvertToClsType(UInt16.MinValue, (Int32)UInt16.MinValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_UInt16_MaxValue()
        {
            TestConvertToClsType(UInt16.MaxValue, (Int32)UInt16.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_UInt32_0()
        {
            TestConvertToClsType((UInt32)0, (Int64)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_UInt32_MinValue()
        {
            TestConvertToClsType(UInt32.MinValue, (Int64)UInt32.MinValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_UInt32_MaxValue()
        {
            TestConvertToClsType(UInt32.MaxValue, (Int64)UInt32.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_UInt64_0()
        {
            TestConvertToClsType((UInt64)0, (Decimal)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_UInt64_MinValue()
        {
            TestConvertToClsType(UInt64.MinValue, (Decimal)UInt64.MinValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_UInt64_MaxValue()
        {
            TestConvertToClsType(UInt64.MaxValue, (Decimal)UInt64.MaxValue);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_UIntPtr_0()
        {
            TestConvertToClsType((UIntPtr)0, (Decimal)0);
        }

        [TestMethod]
        public void ConvertToClsTypeTest_UIntPtr_HighValue()
        {
            TestConvertToClsType((UIntPtr)UInt32.MaxValue, (Decimal)UInt32.MaxValue);
        }

        // ReSharper restore InconsistentNaming
        #endregion
    }
}
