﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Collections.Specialized;
using OpcLabs.BaseLib.OperationModel;

namespace OpcLabs.BaseLibTest.OperationModel
{
    public partial class ValueResultTest
    {
        #region Test Methods
        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void Formatting_F_Failure_ErrorCode_Test()
        {
            Assert.AreEqual("*** Failure: -2 (0xFFFFFFFE): Some error [...] (0 diagnostics); (none) {}", new ValueResult { ErrorMessage = "Some error\nSecond line", ErrorCode = -2 }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Failure_NoErrorCode_Test()
        {
            Assert.AreEqual("*** Failure: 0 (0x00000000): Some error [...] (0 diagnostics); (none) {}", new ValueResult { ErrorMessage = "Some error\nSecond line" }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success 0 (0x00000000):  (4 diagnostics); 1.999 {System.Double}",
                new ValueResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList()),
                    Value = 1.999
                }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Success_ErrorCode_Test()
        {
            Assert.AreEqual("Success 3 (0x00000003):  (0 diagnostics); 1.999 {System.Double}", new ValueResult { ErrorCode = 3, Value = 1.999 }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual("Success 0 (0x00000000):  (0 diagnostics); 1.999 {System.Double}", new ValueResult { Value = 1.999 }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Success_Null_Test()
        {
            Assert.AreEqual("Success 0 (0x00000000):  (0 diagnostics); (none) {}", new ValueResult().ToString("F"));
        }

        [TestMethod]
        public void Formatting_g_Failure_ErrorCode_Test()
        {
            Assert.AreEqual("*** Failure -2: Some error [...]", new ValueResult { ErrorMessage = "Some error\nSecond line", ErrorCode = -2 }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Failure_NoErrorCode_Test()
        {
            Assert.AreEqual("*** Failure: Some error [...]", new ValueResult { ErrorMessage = "Some error\nSecond line" }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success; 1.999",
                new ValueResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList()),
                    Value = 1.999
                }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Success_ErrorCode_Test()
        {
            Assert.AreEqual("Success 3; 1.999", new ValueResult { ErrorCode = 3, Value = 1.999 }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual("Success; 1.999", new ValueResult { Value = 1.999 }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Success_Null_Test()
        {
            Assert.AreEqual("Success; (none)", new ValueResult().ToString("g"));
        }

        [TestMethod]
        public void ToString_Failure_ErrorCode_Test()
        {
            Assert.AreEqual("*** Failure -2 (0xFFFFFFFE): Some error [...]", new ValueResult { ErrorMessage = "Some error\nSecond line", ErrorCode = -2 }.ToString());
        }

        [TestMethod]
        public void ToString_Failure_NoErrorCode_Test()
        {
            Assert.AreEqual("*** Failure: Some error [...]", new ValueResult { ErrorMessage = "Some error\nSecond line" }.ToString());
        }

        [TestMethod]
        public void ToString_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success (4 diagnostics); 1.999 {System.Double}",
                new ValueResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList()),
                    Value = 1.999
                }.ToString());
        }

        [TestMethod]
        public void ToString_Success_ErrorCode_Test()
        {
            Assert.AreEqual("Success 3 (0x00000003); 1.999 {System.Double}", new ValueResult { ErrorCode = 3, Value = 1.999 }.ToString());
        }

        [TestMethod]
        public void ToString_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual("Success; 1.999 {System.Double}", new ValueResult { Value = 1.999 }.ToString());
        }

        [TestMethod]
        public void ToString_Success_Null_Test()
        {
            Assert.AreEqual("Success; (none) {}", new ValueResult().ToString());
        }

        // ReSharper restore InconsistentNaming
        #endregion
    }
}
