﻿using System.Collections.Generic;
using System.Linq;
using JetBrains.Annotations;
using OpcLabs.EasyOpcLiveTest.DataAccess.Environment;

namespace OpcLabs.EasyOpcLiveTest.Legacy.DataAccess
{
    static public class DAServerCatalogEntryExtension
    {
        [NotNull]
        static public IEnumerable<DAItemCatalogEntry> GetConstantItems([NotNull] this DAServerCatalogEntry serverCatalogEntry)
        {
            return serverCatalogEntry.ItemCatalog
                // ReSharper disable once PossibleNullReferenceException
                .Where(entry => entry.BehaviorKind == OpcItemBehaviorKind.Constant);
        }

        [NotNull]
        static public IEnumerable<DAItemCatalogEntry> GetNonReadableItems([NotNull] this DAServerCatalogEntry serverCatalogEntry)
        {
            return serverCatalogEntry.ItemCatalog
                // ReSharper disable once PossibleNullReferenceException
                .Where(entry => !entry.AccessRights.AllowRead);
        }

        [NotNull]
        static public IEnumerable<DAItemCatalogEntry> GetReadableItems([NotNull] this DAServerCatalogEntry serverCatalogEntry)
        {
            return serverCatalogEntry.ItemCatalog
                // ReSharper disable once PossibleNullReferenceException
                .Where(entry => entry.AccessRights.AllowRead);
        }

        [NotNull]
        static public IEnumerable<DAItemCatalogEntry> GetReadAndWriteableItems([NotNull] this DAServerCatalogEntry serverCatalogEntry)
        {
            return serverCatalogEntry.ItemCatalog
                // ReSharper disable once PossibleNullReferenceException
                .Where(entry => entry.AccessRights.AllowRead && entry.AccessRights.AllowWrite);
        }

        [NotNull]
        static public IEnumerable<DAItemCatalogEntry> GetNonWriteableItems([NotNull] this DAServerCatalogEntry serverCatalogEntry)
        {
            return serverCatalogEntry.ItemCatalog
                // ReSharper disable once PossibleNullReferenceException
                .Where(entry => !entry.AccessRights.AllowWrite);
        }

        [NotNull]
        static public IEnumerable<DAItemCatalogEntry> GetWriteableItems([NotNull] this DAServerCatalogEntry serverCatalogEntry)
        {
            return serverCatalogEntry.ItemCatalog
                // ReSharper disable once PossibleNullReferenceException
                .Where(entry => entry.AccessRights.AllowWrite);
        }
    }
}
