﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.Threading;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.OperationModel;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.DataAccess
{
    // ReSharper disable once InconsistentNaming
    public partial class IEasyDAClientTest_Com
    {
#if DEBUG
        private const float ConfigurationFactor = 3.3f;
#else
        private const float ConfigurationFactor = 1.0f;
#endif

        private const float HostFactor = 1.6f;    // test host vs. console app

        [TestCategory("Live")]
        [TestCategory("Performance")]
        [TestCategory("TimeDomain")]
        [TestMethod]  //IMPROVE: put back in; not reliable yet
        // ReSharper disable InconsistentNaming
        public void ReadMultipleItemsTest_Performance()
        // ReSharper restore InconsistentNaming
        {
            const int numberOfItems = 1000;
            const int repeatCount = 10;//2

            EasyDAClient.SharedParameters.TopicParameters.SlowdownWeight = 0.0f;
            EasyDAClient.SharedParameters.TopicParameters.SpeedupWeight = 0.0f;
            IEasyDAClient client = CreateClient();  // just one
            ((EasyDAClient)client).Isolated = true;
            ((EasyDAClient)client).InstanceParameters.Timeouts.ReadItem = 30000;//set 30 seconds timeout for read item
            ((EasyDAClient)client).InstanceParameters.Mode.AllowAsynchronousMethod = false;
            ((EasyDAClient)client).InstanceParameters.UpdateRates.ReadAutomatic = Timeout.Infinite;
            ((EasyDAClient)client).InstanceParameters.UpdateRates.WriteAutomatic = Timeout.Infinite;

            double totalElapsedMilliseconds = 0.0D;
            for (int i = 1; i <= repeatCount; i++)
            {
                // Pause - we do not want the component to use the values it has in memory
                AssertUtilities.WaitingPhase(2 * 1000);

                long elapsedMilliseconds = MeasureReadMultipleItems(() => client, numberOfItems);
                // ReSharper disable LocalizableElement
                Console.WriteLine("elapsedMilliseconds: {0}", elapsedMilliseconds);
                // ReSharper restore LocalizableElement
                totalElapsedMilliseconds += elapsedMilliseconds;
            }

            double averageElapsedMilliseconds = totalElapsedMilliseconds / repeatCount;
            // ReSharper disable LocalizableElement
            Console.WriteLine("averageElapsedMilliseconds: {0}", averageElapsedMilliseconds);
            // ReSharper restore LocalizableElement

            // IMPROVE: Version 5.04 was able to read 2000 items in approx. 500 ms!
            //const double maximumAllowedMilliseconds = 700 * ConfigurationFactor * HostFactor;
            const double maximumAllowedMilliseconds = numberOfItems * 1.4d * ConfigurationFactor * HostFactor;
            // ReSharper disable LocalizableElement
            Console.WriteLine("maximumAllowedMilliseconds: {0}", maximumAllowedMilliseconds);
            // ReSharper restore LocalizableElement

            Assert.IsTrue(averageElapsedMilliseconds <= maximumAllowedMilliseconds);
        }

        private long MeasureReadMultipleItems([NotNull] Func<IEasyDAClient> createClientFunc, int numberOfItems)
        {
            var client = createClientFunc();

            // Create an array of item descriptors for all items
            var itemDescriptors = new DAItemDescriptor[numberOfItems];

            for (int i = 0; i < numberOfItems; i++)
            {
                int copy = (i / 100) + 1;
                int phase = i % 100 + 1;
                string itemId = String.Format(CultureInfo.InvariantCulture, "Simulation.Incrementing.Copy_{0}.Phase_{1}", copy, phase);
                itemDescriptors[i] = itemId;
            }

            // Perform the OPC read
            var stopwatch = new Stopwatch();
            stopwatch.Start();
            ServerDescriptor serverDescriptor = "OPCLabs.KitServer.2";
            //ServerDescriptor serverDescriptor = "OpcTestLab.OpcAnalyzer.1";
            DAVtqResult[] vtqResults = client.ReadMultipleItems(serverDescriptor, itemDescriptors);
            stopwatch.Stop();

            // Count successful results
            int successCount = 0;
            DAVtqResult firstFailedResult = null;
            for (int iItem = 0; iItem < numberOfItems; iItem++)
            {
                DAVtqResult vtqResult = vtqResults[iItem];
                Assert.IsNotNull(vtqResult);
                if (vtqResult.Succeeded)
                    successCount++;
                else if (firstFailedResult == null)
                    firstFailedResult = vtqResult;
            }

            // ReSharper disable LocalizableElement
            Console.WriteLine("successCount: {0}", successCount);
            // ReSharper restore LocalizableElement
            if (firstFailedResult != null)
                Console.WriteLine($"firstFailedResult: {firstFailedResult}");
            Assert.AreEqual(numberOfItems, successCount);

            return stopwatch.ElapsedMilliseconds;
        }
    }
}
