﻿using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.ComInterop;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.AddressSpace;

namespace OpcLabs.EasyOpcLiveTest.Legacy.DataAccess
{
    // ReSharper disable once InconsistentNaming
    public class IEasyDAClientTest_Com_BrowsePath : IEasyDAClientTestBase
    {
        public IEasyDAClientTest_Com_BrowsePath([NotNull] Func<IEasyDAClient> createClientFunc)
            : base(createClientFunc)
        {
        }

        protected void ReadItemAsserts(DAVtq actualVtq, object expectedValue)
        {
            Assert.IsNotNull(actualVtq);

            Assert.IsTrue(actualVtq.Quality.IsGood);
            Assert.AreEqual(DAStatusChoice.GoodNonspecific, actualVtq.Quality.StatusBitField);
            Assert.AreEqual(DALimitChoice.LimitOk, actualVtq.Quality.LimitBitField);

            Assert.AreEqual(DateTimeKind.Utc, actualVtq.Timestamp.Kind);
            Assert.AreNotEqual(actualVtq.Timestamp, DateTime.MinValue);
            Assert.AreNotEqual(actualVtq.Timestamp, DateTime.MaxValue);
            Assert.IsTrue(actualVtq.Timestamp >= DateTime.FromFileTimeUtc(0));
            Assert.IsTrue(actualVtq.Timestamp <= DateTime.Now);

            Assert.AreEqual(expectedValue, actualVtq.Value);
        }

        protected void ReadItemTest([NotNull] ServerDescriptor serverDescriptor, [NotNull] DAItemDescriptor itemDescriptor,
                                    [CanBeNull] object expectedValue)
        {
            var target = CreateClient();
            DAVtq actualVtq = target.ReadItem(serverDescriptor, itemDescriptor);
            ReadItemAsserts(actualVtq, expectedValue);
        }

        protected void ReadItemTest([NotNull] string machineName, [NotNull] string serverClass, [NotNull] string itemId,
            [NotNull] object expectedValue)
        {
            var target = CreateClient();
            DAVtq actualVtq = target.ReadItem(machineName, serverClass, itemId);
            ReadItemAsserts(actualVtq, expectedValue);
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadItemTest_BrowsePath()
        {
            ReadItemTest(
                new ServerDescriptor("", LegacyTestDefs.ProgIds.KitServer),
                new DAItemDescriptor(null, LegacyTestDefs.DataAccess.BrowsePaths.Constant_I4),
                0);
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadItemValueTest_BrowsePath()
        {
            var target = CreateClient();
            object actualValue = target.ReadItemValue(
                new ServerDescriptor("", LegacyTestDefs.ProgIds.KitServer),
                new DAItemDescriptor(null, LegacyTestDefs.DataAccess.BrowsePaths.Constant_I4));
            Assert.AreEqual(0, actualValue);
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void WriteItemValueTest_BrowsePath()
        {
            var target = CreateClient();
            target.WriteItemValue(
                new ServerDescriptor("", LegacyTestDefs.ProgIds.KitServer),
                new DAItemDescriptor(null, LegacyTestDefs.DataAccess.BrowsePaths.Register_BOOL), false);
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowsePropertiesTest_BrowsePath()
        {
            var target = CreateClient();

            DAPropertyElementCollection actualResultCollection = target.BrowseProperties(
                new ServerDescriptor("", LegacyTestDefs.ProgIds.KitServer),
                new DANodeDescriptor(null, LegacyTestDefs.DataAccess.BrowsePaths.Constant_I4));

            Assert.IsNotNull(actualResultCollection);
            Assert.IsTrue(actualResultCollection.Count >= 12);

            //TODO:

        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void GetPropertyValueTest_BrowsePath()
        {
            var target = CreateClient();

            object actualValue = target.GetPropertyValue(
                new ServerDescriptor("", LegacyTestDefs.ProgIds.KitServer),
                new DANodeDescriptor(null, LegacyTestDefs.DataAccess.BrowsePaths.Constant_I4),
                DAPropertyIds.DataType);

            Assert.AreEqual((short)VarTypes.I4, actualValue);

            //TODO:

        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void SubscribeItemTest_BrowsePath()
        {
            IEasyDAClientTest_Com.SubscribeItemTest(
                CreateClient(), 
                new ServerDescriptor("", LegacyTestDefs.ProgIds.KitServer),
                new DAItemDescriptor(null, LegacyTestDefs.DataAccess.BrowsePaths.Incrementing_1s),
                unsubscribeAllItems: false);
        }
    }
}
