﻿using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.Generic;
using OpcLabs.EasyOpc.DataAccess.LiveMapping;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.LiveMapping._Testers
{
    class ItemIdTester
    {
        private class TestRecord
        {
            public int ValueCount { get; set; }
            public int VtqCount { get; set; }
            public int QualityCount { get; set; }
            public int ResultCount { get; set; }
        }

        [DAType, DASubscription(RequestedUpdateRate = 100)]
        private class ItemIdTestType
        {
            public ItemIdTestType([NotNull] TestRecord testRecord)
            {
                TestRecord = testRecord;
            }

            [DANode(ItemId = LegacyTestDefs.DataAccess.ItemIds.Incrementing_1s), DAItem]
            public Int32 Value
            {
                set
                {
                    TestRecord.ValueCount++;
                    AssertUtilities.WriteLine($"Value: {value}");
                }
            }

            [DANode(ItemId = LegacyTestDefs.DataAccess.ItemIds.Incrementing_1s), DAItem(ItemType = typeof(Int32), Kind = DAItemMappingKind.Vtq)]
            public DAVtq<Int32> Vtq
            {
                set
                {
                    TestRecord.VtqCount++;
                    AssertUtilities.WriteLine($"Vtq: {value}");
                }
            }

            [DANode(ItemId = LegacyTestDefs.DataAccess.ItemIds.Incrementing_1s), DAItem(Kind = DAItemMappingKind.Quality)]
            public DAQuality Quality
            {
                set
                {
                    TestRecord.QualityCount++;
                    AssertUtilities.WriteLine($"Quality: {value}");
                }
            }

            [DANode(ItemId = LegacyTestDefs.DataAccess.ItemIds.Incrementing_1s), DAItem(ItemType = typeof(Int32), Kind = DAItemMappingKind.Result)]
            public DAVtqResult<Int32> Result
            {
                set
                {
                    TestRecord.ResultCount++;
                    Assert.IsNotNull(value);
                    AssertUtilities.WriteLine($"Result.TypedVtq: {value.TypedVtq}");
                }
            }

            [NotNull]
            // ReSharper disable MemberCanBePrivate.Local
            public TestRecord TestRecord { get; }
            // ReSharper restore MemberCanBePrivate.Local
        }

        public void Test()
        {
            const int timeout = 10 * 1000;
            const int period = 1000;

            var testRecord = new TestRecord();

            var clientMapper = new DAClientMapper();
            var testTarget = new ItemIdTestType(testRecord);

            clientMapper.Map(testTarget, new DAMappingContext { ServerDescriptor = LegacyTestDefs.ProgIds.KitServer });

            clientMapper.Subscribe(true);
            AssertUtilities.WaitingPhase(timeout);
            clientMapper.Subscribe(false);

            AssertUtilities.WriteLine(testRecord.ValueCount);
            AssertUtilities.WriteLine(testRecord.VtqCount);
            AssertUtilities.WriteLine(testRecord.QualityCount);
            AssertUtilities.WriteLine(testRecord.ResultCount);

            const int minimumCount = timeout/period/2;
            Assert.IsTrue(testRecord.ValueCount >= minimumCount);
            Assert.IsTrue(testRecord.VtqCount >= minimumCount);
            Assert.IsTrue(testRecord.QualityCount >= minimumCount);
            Assert.IsTrue(testRecord.ResultCount >= minimumCount);
        }
    }
}
