﻿
using System.Runtime.InteropServices;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Diagnostics;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.AlarmsAndEvents.OperationModel;
using OpcLabs.EasyOpcFactory.AlarmsAndEvents;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcTest.AlarmsAndEvents
{
    [TestClass]
    public class EasyAENotificationEventArgsTest
    {
        [TestCategory("BinarySerialization")]
        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestMethod]
        public void BinarySerializationTest0()
        {
            Tests.BinarySerializationTest(CreateSerializationTarget0());
        }

        [TestCategory("BinarySerialization")]
        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestMethod]
        public void BinarySerializationTest1()
        {
            Tests.BinarySerializationTest(CreateSerializationTarget1(), SerializationAssertion1);
        }

        [TestCategory("BinarySerialization")]
        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestMethod]
        public void BinarySerializationTest2()
        {
            Tests.BinarySerializationTest(CreateSerializationTarget2(), SerializationAssertion2);
        }

        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestCategory("XmlSerialization")]
        [TestMethod]
        public void XmlSerializationTest0()
        {
            Tests.XmlSerializationTest(CreateSerializationTarget0());
        }

        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestCategory("XmlSerialization")]
        [TestMethod]
        public void XmlSerializationTest1()
        {
            Tests.XmlSerializationTest(CreateSerializationTarget1(), SerializationAssertion1);
        }

        [TestCategory("OpcAlarmsAndEvents")]
        [TestCategory("OpcClassic")]
        [TestCategory("XmlSerialization")]
        [TestMethod]
        public void XmlSerializationTest2()
        {
            Tests.XmlSerializationTest(CreateSerializationTarget2(), SerializationAssertion2);
        }

        [NotNull]
        public EasyAENotificationEventArgs CreateSerializationTarget0()
        {
            return new EasyAENotificationEventArgs();
        }

        [NotNull]
        public EasyAENotificationEventArgs CreateSerializationTarget1()
        {
            return new EasyAENotificationEventArgs
            {
                Arguments = {
                    ServerDescriptor = new ServerDescriptor("TestServerClass"),
                    SubscriptionParameters = AESubscriptionParametersInstances.Instance1
                },
                ErrorCode = 2222,
                Exception = new COMException("TestMessage"),
                AcknowledgedChanged = true,
                EventData = AEEventDataInstances.ConditionInstance1,
                QualityChanged = true,
                RefreshComplete = true,
            };
        }

        [NotNull]
        public EasyAENotificationEventArgs CreateSerializationTarget2()
        {
            return new EasyAENotificationEventArgs
            {
                Arguments = { State = "TestState", }
            };
        }

        public void SerializationAssertion1(EasyAENotificationEventArgs target, EasyAENotificationEventArgs actual)
        {
            Assert.AreEqual(target, actual);
        }

        public void SerializationAssertion2(EasyAENotificationEventArgs target, EasyAENotificationEventArgs actual)
        {
            Check.Assert(target != null);

            Assert.IsNotNull(actual);
            Assert.AreEqual((string)(target.Arguments.State), (string)(actual.Arguments.State));
        }
    }
}
