﻿
using System;
using System.Diagnostics;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Extensions;
using OpcLabs.BaseLib.Security.Cryptography.PkiCertificates;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.Application;
using OpcLabs.EasyOpc.UA.Application.Extensions;
using OpcLabs.EasyOpc.UA.Extensions;
using OpcLabs.UnitTestLib;

namespace OpcLabs.OpcInteroperabilityTest.UA
{
    [TestClass]
    public class IopWS2019_PhoenixTest
    {
        [TestMethod]
        // ReSharper disable once IdentifierTypo
        public void RegisterToGdsTest_MRegen5()
        {
            EasyUAClient.SharedParameters.EngineParameters.CertificateAcceptancePolicy.AcceptAnyCertificate = true;
            // Define which GDS we will work with.
            UAEndpointDescriptor gdsEndpointDescriptor =
                ((UAEndpointDescriptor)"opc.tcp://MREGEN5:58850/GlobalDiscoveryServer")
                // ReSharper disable once StringLiteralTypo
                .WithUserNameIdentity("appadmin", "demo");

            // Obtain the client application service.
            var client = new EasyUAClient();
            IEasyUAClientApplication clientApplication = client.GetService<IEasyUAClientApplication>();
            Debug.Assert(clientApplication != null);

            // Display which application we are about to work with.
            AssertUtilities.WriteLine($"Application URI string: {clientApplication.GetApplicationElement().ApplicationUriString}");

            // Obtain a new application certificate from the certificate manager (GDS), and store it for subsequent usage.
            PkiCertificate certificate = clientApplication.ObtainNewCertificate(gdsEndpointDescriptor,
                new Progress<string>(s => Console.WriteLine("Progress: {0}", s)));

            // Display results
            AssertUtilities.WriteLine($"Certificate: {certificate}");
        }
    }
}
