﻿

using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace;
using OpcLabs.EasyOpc.UA.AddressSpace.Standard;
using OpcLabs.EasyOpc.UA.Extensions;
using OpcLabs.EasyOpc.UA.Graphs;
using OpcLabs.UnitTestLib;

namespace OpcLabs.OpcInteroperabilityTest.UA
{
    partial class UAInteroperabilityWorkshopTest
    {
        #region Test Methods

        [TestMethod]
        public void Group103_ViewServices2_Header____________________()
        {
        }

        [TestMethod]
        public void Group103_ViewServices2_Test01_Browse_Variables()
        {
            TestSection.Arrange();
            PrecheckTestMethod();
            UAEndpointDescriptor endpointDescriptor = EndpointDescriptorCommon;

            TestSection.Act();
            using (var client = new EasyUAClient { Isolated = true })
            {
                var browseParameters = new UABrowseParameters(UANodeClass.Variable);
                UANodeElementCollection nodeElementCollection = client.Browse(endpointDescriptor, UAObjectIds.ObjectsFolder, browseParameters);
                foreach (UANodeElement nodeElement in nodeElementCollection)
                    // ReSharper disable once PossibleNullReferenceException
                    AssertUtilities.WriteLine($"{nodeElement.DisplayName}: {nodeElement}");
                AssertUtilities.WriteLine($"Element count: {nodeElementCollection.Count}");

                if (nodeElementCollection.Count == 0)
                {
                    const int count = 100;
                    IEnumerable<UATreePosition> treePositions = client.TraverseNodeTree(endpointDescriptor);
                    UATreePosition variablePosition = treePositions
                            .Take(count)
                            // ReSharper disable once PossibleNullReferenceException
                            .FirstOrDefault(position => position.NodeClass == UANodeClass.Variable);
                    Assert.IsNotNull(variablePosition, "Found no variables");

                    UANodeDescriptor parentNodeDescriptor = variablePosition.ParentNodeDescriptor;
                    AssertUtilities.WriteLine($"Found parent node: {parentNodeDescriptor}");
                    Debug.Assert(parentNodeDescriptor != null);
                    nodeElementCollection = client.Browse(endpointDescriptor, parentNodeDescriptor, UABrowseParameters.AllForwardReferences);
                    foreach (UANodeElement nodeElement in nodeElementCollection)
                        // ReSharper disable once PossibleNullReferenceException
                        AssertUtilities.WriteLine($"{nodeElement.DisplayName}: {nodeElement}");
                    AssertUtilities.WriteLine($"Element count: {nodeElementCollection.Count}");

                    TestSection.Assert();
                    Assert.IsTrue(nodeElementCollection.Count >= 1, "No elements");
                }
            }
        }

        #endregion
    }
}
